#include <u.h>
#include <libc.h>
#include <bio.h>

char *Systype[] = {
	[0] "unknown",
	[1] "native",
	[2] "Win32 GUI",
	[3] "Win32 console",
	[5] "OS/2 console",
	[7] "Posix/SFU",
	[8] "Win9x",
	[9] "WinCE GUI",
	[10] "EFI app",
	[11] "EFI boot driver",
	[12] "EFI runtime driver",
	[13] "EFI ROM",
	[14] "Xbox",
};

vlong
gint(Biobuf *bp, uvlong off, int len)
{
	int i, c;
	uvlong vl, rc;

	rc = 0;
	Bseek(bp, off, 0);
	for (i = 0; i < len; i++){
		if ((c = Bgetc(bp)) == -1)
			sysfatal("unexpected EOF - %r\n");
		vl = c;
		rc |= vl << (8*i);
	}
	return rc;
}

int
exetype(char *file)
{
	Biobuf *bp;
	int off, magic, subsys;

	if ((bp = Bopen(file, OREAD)) == nil){
		fprint(2, "%s %s cannot open %r\n", argv0, file);
		return -1;
	}

	magic = gint(bp, 0, 2);
	if (magic != 0x5A4D){
		print("%-24s not a windows .exe\n", file);
		return -1;
	}

	magic = gint(bp, 0x18, 1);
	if(magic != 0x40){
		print("%-24s MSDOS\n", file);
		return 0;
	}

	off = gint(bp, 0x3c, 4);
	magic = gint(bp, off, 2);
	switch(magic){
	case 0x4550:
		// Win32
		break;
	case 0x454E:
		print("%-24s O/S2 16bit\n", file);
		exits(nil);
		break;
	case 0x454C:
		print("%-24s O/S2 32bit\n", file);
		exits(nil);
	default:
		print("%-24s unknown .EXE magic\n", file);
		exits(nil);
	}

	subsys = gint(bp, off+0x5c, 2);
	if(subsys < 0 || subsys >= nelem(Systype))
		print("%-24s %d\n", file, subsys);
	else
		print("%-24s %s\n", file, Systype[subsys]);
	return 0;
}

void
main(int argc, char *argv[])
{
	int rc;

	rc = 0;
	argv0 = argv[0];
	while(argv++, --argc)
		rc |= exetype(*argv);
	exits((rc == 0)? nil: "failed");
}
